/*
 * Copyright (c) 2009, Microsoft Corporation - All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * LICENSE-GPL in the main directory of this source tree, or the
 * BSD license (http://opensource.org/licenses/bsd-license.php).
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Authors:
 *   Haiyang Zhang <haiyangz@microsoft.com>
 *   Hank Janssen  <hjanssen@microsoft.com>
 */
#ifndef _RING_BUFFER_H_
#define _RING_BUFFER_H_

#include "osd.h"

typedef struct _SG_BUFFER_LIST {
	PVOID	Data;
	UINT32	Length;
} SG_BUFFER_LIST;

typedef struct _RING_BUFFER {
    volatile UINT32	WriteIndex;     // Offset in bytes from the start of ring data below
    volatile UINT32	ReadIndex;      // Offset in bytes from the start of ring data below

	volatile UINT32 InterruptMask;
	UINT8	Reserved[4084];			// Pad it to PAGE_SIZE so that data starts on page boundary
	// NOTE: The InterruptMask field is used only for channels but since our vmbus connection
	// also uses this data structure and its data starts here, we commented out this field.
	// volatile UINT32 InterruptMask;
	// Ring data starts here + RingDataStartOffset !!! DO NOT place any fields below this !!!
    UINT8		Buffer[0];
} STRUCT_PACKED RING_BUFFER;

typedef struct _RING_BUFFER_INFO {
    RING_BUFFER*	RingBuffer;
    UINT32			RingSize;			// Include the shared header
	HANDLE			RingLock;

    UINT32			RingDataSize;		// < ringSize
	UINT32			RingDataStartOffset;

} RING_BUFFER_INFO;


typedef struct _RING_BUFFER_DEBUG_INFO {
	UINT32		CurrentInterruptMask;
	UINT32		CurrentReadIndex;
	UINT32		CurrentWriteIndex;
	UINT32		BytesAvailToRead;
	UINT32		BytesAvailToWrite;
}RING_BUFFER_DEBUG_INFO;


//
// Interface
//

INTERNAL int
RingBufferInit(
	RING_BUFFER_INFO	*RingInfo,
	PVOID				Buffer,
	UINT32				BufferLen
	);

INTERNAL void
RingBufferCleanup(
	RING_BUFFER_INFO	*RingInfo
	);

INTERNAL int
RingBufferWrite(
	RING_BUFFER_INFO	*RingInfo,
	SG_BUFFER_LIST		SgBuffers[],
	UINT32				SgBufferCount
	);

INTERNAL int
RingBufferPeek(
	RING_BUFFER_INFO	*RingInfo,
	PVOID				Buffer,
	UINT32				BufferLen
	);

INTERNAL int
RingBufferRead(
	RING_BUFFER_INFO	*RingInfo,
	PVOID				Buffer,
	UINT32				BufferLen,
	UINT32				Offset
	);

INTERNAL UINT32 
GetRingBufferInterruptMask(
	RING_BUFFER_INFO *RingInfo
	);

INTERNAL void
DumpRingInfo(
	RING_BUFFER_INFO* RingInfo, 
	char *Prefix
	);

INTERNAL void
RingBufferGetDebugInfo(
	RING_BUFFER_INFO		*RingInfo,
	RING_BUFFER_DEBUG_INFO	*DebugInfo
	);

#endif // _RING_BUFFER_H_
