/*
 * Copyright (c) 2009, Microsoft Corporation - All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * LICENSE-GPL in the main directory of this source tree, or the
 * BSD license (http://opensource.org/licenses/bsd-license.php).
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Authors:
 *   Haiyang Zhang <haiyangz@microsoft.com>
 *   Hank Janssen  <hjanssen@microsoft.com>
 */
#include "timesync_ic.h"

void timesync_channel_cb(void *context){
        VMBUS_CHANNEL *channel = context;
	u8 *buf;
	u32 buflen, recvlen;
	u64 requestid;
	struct icmsg_hdr *icmsghdrp;
	struct icmsg_negotiate *negop;
	struct ictimesync_data *timedatap;

	DPRINT_ENTER(VMBUS);

	buflen = PAGE_SIZE;
	buf = MemAllocAtomic(buflen);

	VmbusChannelRecvPacket(channel, buf, buflen, &recvlen, &requestid);

	if(recvlen > 0) {
	    DPRINT_DBG(VMBUS, "timesync packet: recvlen=%d, requestid=%ld", 
			recvlen, requestid);

	    icmsghdrp = (struct icmsg_hdr *)&buf[sizeof(struct vmbuspipe_hdr)];
	    

	    if(icmsghdrp->icmsgtype == ICMSGTYPE_NEGOTIATE) {
		icmsghdrp->icmsgsize = 0x10;
		negop = (struct icmsg_negotiate *)&buf[
		    sizeof(struct vmbuspipe_hdr) +
		    sizeof(struct icmsg_hdr)];
		if(negop->icframe_vercnt == 2 &&
		   negop->icversion_data[1].major == 3) {
		    negop->icversion_data[0].major = 3;
		    negop->icversion_data[0].minor = 0;
		    negop->icversion_data[1].major = 3;
		    negop->icversion_data[1].minor = 0;
		} else {
		    negop->icversion_data[0].major = 1;
		    negop->icversion_data[0].minor = 0;
		    negop->icversion_data[1].major = 1;
		    negop->icversion_data[1].minor = 0;
		}
		negop->icframe_vercnt = 1;
		negop->icmsg_vercnt = 1;
	    } else {
		timedatap = (struct ictimesync_data *)&buf[
		    sizeof(struct vmbuspipe_hdr) +
		    sizeof(struct icmsg_hdr)];
		adj_guesttime(timedatap->parenttime, timedatap->flags);
	    }

	    icmsghdrp->icflags = ICMSGHDRFLAG_TRANSACTION 
		| ICMSGHDRFLAG_RESPONSE;

	    VmbusChannelSendPacket(channel, buf,
				   recvlen, requestid,
				   VmbusPacketTypeDataInBand, 0);
	}

	MemFree(buf);

	DPRINT_EXIT(VMBUS);
}

