/*
 * Copyright (c) 2009, Microsoft Corporation - All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * LICENSE-GPL in the main directory of this source tree, or the
 * BSD license (http://opensource.org/licenses/bsd-license.php).
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Authors:
 *   Haiyang Zhang <haiyangz@microsoft.com>
 *   Hank Janssen  <hjanssen@microsoft.com>
 */
#ifndef _NETVSC_H_
#define _NETVSC_H_

#include "VmbusPacketFormat.h"
#include "nvspprotocol.h"

#include "List.h"

#include "NetVscApi.h"
//
// #defines
//
//#define NVSC_MIN_PROTOCOL_VERSION                       1
//#define NVSC_MAX_PROTOCOL_VERSION                       1

#define NETVSC_SEND_BUFFER_SIZE				64*1024 // 64K
#define NETVSC_SEND_BUFFER_ID				0xface


#define NETVSC_RECEIVE_BUFFER_SIZE			1024*1024 // 1MB

#define NETVSC_RECEIVE_BUFFER_ID			0xcafe

#define NETVSC_RECEIVE_SG_COUNT				1

// Preallocated receive packets
#define NETVSC_RECEIVE_PACKETLIST_COUNT		256

//
// Data types
//

// Per netvsc channel-specific
typedef struct _NETVSC_DEVICE {
	DEVICE_OBJECT					*Device;

	int								RefCount;

	int								NumOutstandingSends;
	// List of free preallocated NETVSC_PACKET to represent receive packet
	LIST_ENTRY						ReceivePacketList;
	HANDLE							ReceivePacketListLock;

	// Send buffer allocated by us but manages by NetVSP
	PVOID							SendBuffer;
	UINT32							SendBufferSize;
	UINT32							SendBufferGpadlHandle;
	UINT32							SendSectionSize;

	// Receive buffer allocated by us but manages by NetVSP
	PVOID							ReceiveBuffer;
	UINT32							ReceiveBufferSize;
	UINT32							ReceiveBufferGpadlHandle;
	UINT32							ReceiveSectionCount;
	PNVSP_1_RECEIVE_BUFFER_SECTION	ReceiveSections;

	// Used for NetVSP initialization protocol
	HANDLE							ChannelInitEvent;
	NVSP_MESSAGE					ChannelInitPacket;

	NVSP_MESSAGE					RevokePacket;
	//UCHAR							HwMacAddr[HW_MACADDR_LEN];

	// Holds rndis device info
	void							*Extension;
} NETVSC_DEVICE;

#endif // _NETVSC_H_
